-----------------------------------------------------------------------------------------
-- BASIC LUA/CORONA FUNCTIONS
-----------------------------------------------------------------------------------------
clear = cleanImg
testMode = false
--
currentOrientation = "landscapeLeft"
function onOrientationChange( event )
	if event.type == "landscapeLeft" or event.type == "landscapeRight" then
		currentOrientation = event.type
	end
end
Runtime:addEventListener( "orientation", onOrientationChange )
--
function isInArray(value, t)
	if #t > 0 then
        for n = 1, #t do
            if t[n] == value then
                return true
            end
        end
	end
return false
end
function isInArrayAdv(value, t)
	if #t > 0 then
        for n = 1, #t do
			if t[n][1] == value then
				return true
			end
        end
	end
return false
end
--
img_array = {}
for i = 1, 64 do
	img_array[i] = {}
end
timer_array = {}
for i = 1, 8 do
	timer_array[i] = {}
end
arrows = {}
for i = 1, 64 do
	arrows[i] = 0
end
textFields = {}
for i = 1, 128 do
	textFields[i] = ""
end
--
function doTimer( delay, f, params, scene )
	local a = timer.performWithDelay( delay, f )
	a.params = params
	if scene == nil then scene = 1 end
	timer_array[scene][#timer_array[scene]+1] = a
end
function cancelTimers( scene )
	for k,v in pairs(timer_array[scene]) do
		timer.cancel(v)
	end
end
function displayNewImage( path, x, y, scene )
	if scene == nil then
		scene = 1
	end
	local num = #img_array[scene]+1
	img_array[scene][num] = display.newImage( path, x, y )
	return img_array[scene][num]
end
--
function displayNewText( text, x, y, font, size, scene, align )
	if scene == nil then
		scene = 1
	end
	if align == nil then
		align = "left"
	end
	if text ~= nil then
		local num = #img_array[scene]+1
		img_array[scene][num] = display.newText( {text = text, x = x, y = y, font = "Chinacat", fontSize = size, align = align} )
		return img_array[scene][num]
	end
end
--
function displayNewRect( xc, yc, x, y, scene )
	if scene == nil then
		scene = 1
	end
	local num = #img_array[scene]+1
	img_array[scene][num] = display.newRect( x, y, xc, yc )
	return img_array[scene][num]
end
--
function cleanImg( scen )
	if scen == nil then
		scene = {1, #img_array}
	else
		scene = {scen, scen}
	end
	if #img_array > 0 then
		for i = scene[1], scene[2] do
			if #img_array[i] > 0 then
				for k,v in pairs(img_array[i]) do
					img_array[i][k]:removeSelf()
					img_array[i][k] = nil
				end
			end
			img_array[i] = nil
			img_array[i] = {}
		end
	end
	if testMode == true then
		local l = 0
		for i = 1, #img_array do
			l = l + #img_array[i]
		end
		print(">> Cleaned... "..l.." objects left in ".. #img_array .." scenes")
	end
end
--
local clock = os.clock
function sleep(n)
  local t0 = clock()
  while clock() - t0 <= n do end
end
--
function vibrate()
	if config["Vibrations"] == 1 then
		system.vibrate()
	end
end
--
function gui( sizex, sizey, xc, yc, title, style, scene )
	if scene == nil then scene = 1 end
	local img_size = {320, 160}
	local disp = displayNewImage( "img/gui_"..img_size[1].."x"..img_size[2].."_"..style..".png", xc, yc, scene)
	if disp ~= nil then
		displayNewText( title, xc, (yc-(sizey/2)+2)+12, "Arial", 22, scene, "center" )
		disp:scale( sizex/img_size[1], sizey/img_size[2] )
	end
	return disp
end
--
function newButton( x, y, text, fontSize, font, whatDo, parameters, sizeX, sizeY)
	if sizeX == nil then
		sizeX = 256
	end
	if sizeY == nil then
		sizeY = fontSize+(2*6)
	end
	
	--[[local a = displayNewImage( "img/button_2.png", x, y)
	local b = displayNewImage( "img/button_1.png", x, y)
	a:scale( 1, 0.8 )
	b:scale( 1, 0.8 )]]--
	gui( sizeX, sizeY, x, y, "", 2 )
	local b = gui( sizeX, sizeY, x, y, "", 1 )
	b.f = whatDo
	b.p = parameters
	local t = displayNewText( text, x, y, font, fontSize)
	--t:setFillColor(0.5,0.5,0.5)
	if whatDo ~= nil then
		b:addEventListener( "touch", altButton)
	end
	return {b, t}
end
--
function isInObject( object, posx, posy )
	local x = math.ceil(object.contentWidth/2)-6
	local y = math.ceil(object.contentHeight/2)-6
	local xc, yc = object:localToContent( 0, 0 )
	if posx > xc-x and posx < xc+x then
		if posy > yc-y and posy < yc+y then
			return true
		end
	end
	return false
end
--
function altButton( event )
	if event.phase == "ended" and isInObject( event.target, event.x, event.y ) == true then
		menu = 0
		inSpells = 0
		event.target.f(event.target.p[1], event.target.p[2], event.target.p[3], event.target.p[4], event.target.p[5], event.target.p[6], event.target.p[7], event.target.p[8], event.target.p[9], event.target.p[10])
	elseif isInObject( event.target, event.x, event.y ) == false then
		transition.to( event.target, { time=100, alpha=1 } )
	elseif isInObject( event.target, event.x, event.y ) == true then
		transition.to( event.target, { time=100, alpha=0.1 } )
	end
end
--
function removefromtable(v, t)
	if #t > 0 then
		for i = 1, #t do
			if v == t[i] then
				table.remove(t, i)
			end
		end
	end
end
function removefromtableadv(v, t)
	if #t > 0 then
		for key, val in pairs(t) do
			if v == t[key][1] then
				table.remove(t, key)
				return true
			end
		end
	end
	return false
end
--
function arrow( event )
	if event.phase == "ended" then
		if event.target.v ~= nil then
			hcc[4][event.target.v] = hcc[4][event.target.v]+event.target.n
			event.target.f()
		elseif event.target.vv ~= nil then
			arrows[event.target.vv] = arrows[event.target.vv]+event.target.n
			event.target.f()
		else
			hcc[1] = hcc[1]+event.target.n
			event.target.f()
		end
	end
end
function displayArrows(x, y, sizex, f, id, maxv, color)
	if maxv < 0 then
		maxv = 0
	end
	local b = displayNewImage( "img/arrow.png", x+((-sizex)*36), y, 3 )
	b.f = f
	b.n = -1
	b.vv = id
	if color ~= nil then
		b:setFillColor(color[1],color[2],color[3])
	end
	if arrows[id] <= 0 then
		b.alpha = 0.5
	else
		b:addEventListener("touch", arrow)
	end
	local b = displayNewImage( "img/arrow.png", x+((sizex)*36), y, 3 )
	b.f = f
	b.n = 1
	b.vv = id
	b.xScale = -1
	if color ~= nil then
		b:setFillColor(color[1],color[2],color[3])
	end
	if maxv <= arrows[id] then
		b.alpha = 0.5
	elseif f ~= nil then
		b:addEventListener("touch", arrow)
	end
end
--
function getAngle( x, y, cx, cy )
	local angle = 0
	if x >= cx and y <= cy then
		local cosa = (cy-y)/math.sqrt(math.pow(x-cx,2)+math.pow(cy-y,2))
		angle = math.deg(math.acos(cosa))
	elseif x >= cx and y >= cy then
		local cosa = (x-cx)/math.sqrt(math.pow(x-cx,2)+math.pow(y-cy,2))
		angle = math.deg(math.acos(cosa))+90
	elseif x <= cx and y >= cy then
		local cosa = (y-cy)/math.sqrt(math.pow(cx-x,2)+math.pow(y-cy,2))
		angle = math.deg(math.acos(cosa))+180
	elseif x <= cx and y <= cy then
		local cosa = (cx-x)/math.sqrt(math.pow(cx-x,2)+math.pow(cy-y,2))
		angle = math.deg(math.acos(cosa))+270
	end
	return angle
end
--
function deepcopy(orig)
local orig_type = type(orig)
local copy
	if orig_type == 'table' then
		copy = {}
		for orig_key, orig_value in next, orig, nil do
			copy[deepcopy(orig_key)] = deepcopy(orig_value)
		end
		setmetatable(copy, deepcopy(getmetatable(orig)))
	else -- number, string, boolean, etc
		copy = orig
	end
	return copy
end
--
function table.tostring( tbl )
  local result, done = {}, {}
  for k, v in ipairs( tbl ) do
    table.insert( result, table.val_to_str( v ) )
    done[ k ] = true
  end
  for k, v in pairs( tbl ) do
    if not done[ k ] then
      table.insert( result,
        table.key_to_str( k ) .. "=" .. table.val_to_str( v ) )
    end
  end
  return "{" .. table.concat( result, "," ) .. "}"
end
--
function textListener( event )-- for displayNewTextField
	if ( event.phase == "ended" or event.phase == "submitted" ) then
		textFields[event.target.idd] = event.target.text
    end
end
function displayNewTextField( cx, cy, sx, sy, text, id, scene ) -- displayNewTextField( centerX, centerY, width, height, ID )
	if scene == nil then
		scene = 1
	end
	local num = #img_array[scene]+1	
	img_array[scene][num] = native.newTextField( cx, cy, sx, sy )
	img_array[scene][num]:addEventListener( "userInput", textListener )
	img_array[scene][num].size = sy*0.4
	img_array[scene][num].idd = id
	img_array[scene][num].text = text
	--img_array[scene][num].placeholder = text
	print(text)
	return img_array[scene][num]
end
function firstToUpper(str)
    return (str:gsub("^%l", string.upper))
end
--
function playSound( file )
	if loadTable("cfg")["Sounds"] == 1 then
		audio.setVolume( 1, {channel=1} )
		audio.play( audio.loadSound( "sounds/"..file, {channel=1} ) )
	end
end
-----------------------------------------------------------------------------------------
-- FOR GAME BASIC FUNCTIONS
-----------------------------------------------------------------------------------------
function reloadHeroes()-- reload heroes table
	heroes = nil 
	heroes = deepcopy(heroes__)
	local n = #heroes
	local hero_array = loadTable("heroes")
	if hero_array ~= nil then
		for i = 1, #hero_array do
			heroes[n+i] = hero_array[i]
		end
	end
end
--
function reloadMaps()-- reload maps table
	for k, v in pairs(a_maps) do
		if loadTable(k) == nil then
			local tmp_cities = {}
			local me_map = a_maps[k][1]
			for x = 1, #me_map do
				for y = 1, #me_map[x] do
					if me_map[x][y][3] == 1 then
						local c = {owner = me_map[x][y][6], cityid = me_map[x][y][4], country = "polandball", buildings = {"1", "m"}, buyable = {8, 6, 5, 3, 1}}
						table.insert(tmp_cities, c)
					end
				end
			end
			saveTable({me_map,tmp_cities,a_maps[k][3]},k)
		end
	end
	maps = nil 
	maps = deepcopy(maps_)
	local n = #maps
	local map_array = loadTable("maps")
	if map_array ~= nil then
		for i = 1, #map_array do
			maps[n+i] = map_array[i]
		end
	end
end
--
function checkBalance()
	--print(">> >> BALANCING MATH!")
	for k, v in pairs (countries) do
		print(">> Now: ".. k)
		local statsSumm = {0,0,0,0,0}
		
		local pr = {{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}}
		local prN = {{100, 0}, {250, 0}, {350, 0}, {600, 5}, {1000, 15}}
		for i = 1, #v[1] do
			local s = v[1][i]
			if s.att+s.def+s.acc+s.sp ~= 20+10*i then
				statsSumm[i] = s.att+s.def+s.acc+s.sp
			end
			if s.cost[1] ~= prN[i][1] or s.cost[2] ~= prN[i][2] then
				pr[i] = {s.cost[1], s.cost[2]}
			end
		end
		local stats = ""
		for i = 1, 5 do
			if statsSumm[i] ~= 0 then
				stats = stats.." ["..i.."] = "..statsSumm[i]..","
			end
		end
		local price = ""
		for i = 1, 5 do
			if (pr[i][1] ~= nil and pr[i][1] ~= 0) or (pr[i][1] ~= nil and pr[i][2] ~= 0) then
				price = price.." ["..i.."] = {"..pr[i][1]..", "..pr[i][2].."},"
			end
		end
		if stats ~= "" or price ~= "" then
			print("> Units")
		end
		if stats ~= "" then
			print("Wrong stats:")
			print(stats)
		end
		if price ~= "" then
			print("Wrong prices:")
			print(price)
		end
		local pr = {{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}}
		local prN = {["1"] = {350, 30}, ["2"] = {500, 50}, ["3"] = {1000, 100}, ["4"] = {2200, 150}, ["5"] = {4000, 200}, ["m"] = {1000, 100}}
		for k1, v1 in pairs (v[2]) do
			local s = v1[4]
			if s[1] ~= prN[k1][1] or s[2] ~= prN[k1][2] then
				pr[k1] = {s[1], s[2]}
			end
		end
		local price = ""
		for i = 1, 5 do
			if pr[i][1] ~= 0 or pr[i][2] ~= 0 then
				price = price.." ["..i.."] = {"..pr[i][1]..", "..pr[i][2].."},"
			end
		end
		if price ~= "" then
			print("> Buildings")
			print("Wrong prices:")
			print(price)
		end
		if price == "" then
		end
	end
end
--
function checkBalanceAdvance( country, detailed ) 
	local balance = {
		[1] = 0, 
		[2] = 0, 
		[3] = 0, 
		[4] = 0,
		[5] = 0,
		}
	local a = 0
	for un, u in pairs (countries[country][1]) do
		for k, v in pairs (countries) do
			if #v[1] == 5 then
				for i = 1, #v[1] do
					a = a +1
					local con1, con2, con3, con4, con5 = CONST_ATT, CONST_DEF, CONST_TIER, CONST_SPD, CONST_ARCH
					-- con1 - const attack, con2 - const def, con3 - tier const, con4 - const speed, con5 - archer damage const
					local en = countries[k][1][i]
					-- \/ calculators
					local deal = math.ceil(((u.att*(con1+(10/100)))-(en.def*(con2+(10/100))))/(un/con3))
					if isInArray(un, archers[country]) == true then
						deal = deal*con5
					end
					local rece = math.ceil(((en.att*(con1+(10/100)))-(u.def*(con2+(10/100))))/(i/con3))
					if isInArray(i, archers[k]) == true then
						rece = rece*con5
					end
					if deal < 1 then
						deal = 1
					end
					if rece < 1 then
						rece = 1
					end
					local him = (u.acc-math.floor(en.sp/con4))*0.1
					local me = (en.acc-math.floor(u.sp/con4))*0.1
					
					balance[un] = balance[un] + (deal*him) - (rece*me)
				end
			end
		end
		balance[un] = balance[un]/a
		a = 0
	end
	if detailed ~= true then
		local v = 0
		for i = 1, 5 do
			v = v+balance[i]
		end
		v = v/5
		print("["..country.."]	  "..v)
	else
		for i = 1, 5 do
			print("["..country.."] Unit ".. i .." - "..balance[i])
		end
	end
end
--
function balanceCheckFull()
	print("----------")
	print("-- Part 1")
	print("----------")
	print("-- Checking if all prices and stats are on proper level")
	checkBalance()
	print("----------")
	print("-- Part 2")
	print("----------")
	print("-- Checking damage ratio = (damage dealt * chance to hit) - (damage received * chance to get hit)")
	for k,v in pairs (countries) do
		if #v[1] >= 5 then
			checkBalanceAdvance( k )
		end
	end
end
--
function getHeroBonus( hid, expanded )
	local bonus = {}
	if expanded == true then
		bonus = {
		uAtt = 0, uDef = 0, uAcc = 0, uSpd = 0,
		att = 0, def = 0, int = 0,
		-- 
		}
	end
	for k, v in pairs(sc_heroes) do
		if v.hid == hid and #v.items > 0 then
			for i = 1, v.items do
				if expanded ~= nil and expanded == true then
					local it = items[v.items[i]][2]
					bonus = {
						uAtt = bonus.uAtt+(it.uAtt or 0), uDef = bonus.uDef+(it.uDef or 0), uAcc = bonus.uAcc+(it.uAcc or 0), uSpd = bonus.uSpd+(it.uSpd or 0),
						att = bonus.att+(it.att or 0), def = bonus.def+(it.def or 0), int = bonus.int+(it.int or 0),
					-- 
					}
				else
					local id = items[v.items[i]][2]
					for k1, v1 in pairs(id) do
						bonus.k1 = (bonus.k1 or 0)+v1
					end
				end
			end
		end
	end
	return bonus
end
-----------------------------------------------------------------------------------------
-- POPUPS
-----------------------------------------------------------------------------------------
function popupFull()
	local tpopup3 = "This game is RNG based, as it\nmeant to be. Don't take it too serious!\n\nThis game is not intended\nto offend anyone."
	local function popup3()
		cleanImg()
		local r3 = displayNewRect( display.contentCenterX*2, display.contentCenterY*2, display.contentCenterX, display.contentCenterY, 62 )
		local t3 = displayNewText( tpopup3, display.contentCenterX, display.contentCenterY, "Arial", 24, 62, "center" )
		r3:setFillColor(0,0,0)
		
		transition.to( t3, {time=500, delay=5000, alpha=0, tag="popup"} )
		transition.to( r3, {time=500, delay=5000, alpha=0, tag="popup", onComplete=menuDs} )
		local function touchPopup(event)
			if event.phase == "ended" then
				transition.cancel("popup")
				transition.to( r3, {time=250, alpha=0, tag="popup"} )
				transition.to( t3, {time=250, alpha=0, tag="popup", onComplete=menuDs} )
			end
		end
		r3:addEventListener("touch", touchPopup)
	end
	
	local function popup2()
		cleanImg()
		
		local r3 = displayNewRect( display.contentCenterX*2, display.contentCenterY*2, display.contentCenterX, display.contentCenterY, 62 )
		local t3 = displayNewText( tpopup3, display.contentCenterX, display.contentCenterY, "Arial", 24, 62, "center" )
		r3:setFillColor(0,0,0)
		
		local r2 = displayNewRect( display.contentCenterX*2, display.contentCenterY*2, display.contentCenterX, display.contentCenterY, 62 )
		local t2 = displayNewImage( "popup_2.png", display.contentCenterX, display.contentCenterY, 62 )
		
		r2:setFillColor(1,1,1)
		transition.to( t2, {time=500, delay=3000, alpha=0, tag="popup"} )
		transition.to( r2, {time=500, delay=3000, alpha=0, tag="popup", onComplete=popup3} )
		local function touchPopup(event)
			if event.phase == "ended" then
				transition.cancel("popup")
				transition.to( r2, {time=250, alpha=0, tag="popup"} )
				transition.to( t2, {time=250, alpha=0, tag="popup", onComplete=popup3} )
			end
		end
		r2:addEventListener("touch", touchPopup)
	end
	
	local function popup1()
		cleanImg()
		
		local r2 = displayNewRect( display.contentCenterX*2, display.contentCenterY*2, display.contentCenterX, display.contentCenterY, 62 )
		local t2 = displayNewImage( "popup_2.png", display.contentCenterX, display.contentCenterY, 62 )
		
		local t1 = displayNewImage( "popup_1.jpg", display.contentCenterX, display.contentCenterY, 62 )
		transition.to( t1, {time=500, delay=1500, alpha=0, tag="popup", onComplete=popup2} )
		local function touchPopup(event)
			if event.phase == "ended" then
				transition.cancel("popup")
				transition.to( t1, {time=250, alpha=0, tag="popup", onComplete=popup2} )
			end
		end
		t1:addEventListener("touch", touchPopup)
	end
	
	popup1()
end