-----------------------------------------------------------------------------------------
-- SAVE-LOAD FUNCTIONS
-----------------------------------------------------------------------------------------
local json = require("json")
function saveTable(t, filename)
    local path = system.pathForFile( filename, system.DocumentsDirectory)
    local file = io.open(path, "w")
    if file then
        local contents = json.encode(t)
        file:write( contents )
        io.close( file )
        return true
    else
        return false
    end
end
--
function loadTable(filename)
    local path = system.pathForFile( filename, system.DocumentsDirectory)
    local contents = ""
    local myTable = {}
    local file = io.open( path, "r" )
    if file then
         -- read all contents of file into a string
         local contents = file:read( "*a" )
         myTable = json.decode(contents);
         io.close( file )
         return myTable 
    end
    return nil
end
--
function table.val_to_str ( v )
  if "string" == type( v ) then
    v = string.gsub( v, "\n", "\\n" )
    if string.match( string.gsub(v,"[^'\"]",""), '^"+$' ) then
      return "'" .. v .. "'"
    end
    return '"' .. string.gsub(v,'"', '\\"' ) .. '"'
  else
    return "table" == type( v ) and table.tostring( v ) or
      tostring( v )
  end
end
function table.key_to_str ( k )
  if "string" == type( k ) and string.match( k, "^[_%a][_%a%d]*$" ) then
    return k
  else
    return "[" .. table.val_to_str( k ) .. "]"
  end
end
function table.tostring( tbl )
  local result, done = {}, {}
  for k, v in ipairs( tbl ) do
    table.insert( result, table.val_to_str( v ) )
    done[ k ] = true
  end
  for k, v in pairs( tbl ) do
    if not done[ k ] then
      table.insert( result,
        table.key_to_str( k ) .. "=" .. table.val_to_str( v ) )
    end
  end
  return "{" .. table.concat( result, "," ) .. "}"
end
function saveToTxt( txt1, filename )
	local txt = table.tostring( txt1 )
	local path = system.pathForFile( filename, system.DocumentsDirectory )
	file = io.open(path, "w")
	file:write(txt)
	file:close()
end
function saveMapTxt( mapName )
	local mtocop, mresult = loadTable("map_"..mapName), {{}, {}}
	for i = 1, 32 do 
		mresult[1][i] = {}
		for j = 1, 32 do
			mresult[1][i][j] = mtocop[1][i][j]
		end
	end
	mresult[2] = mtocop[2]
	mresult[3] = mtocop[3]
	saveToTxt( mresult, mapName.."_intxt.txt" )
end